@@ -1,16 +1,16 @@
+#include "array.h"
+#include "fastcgi.h"
+
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "array.h"
-#include "fastcgi.h"
-
 static data_unset *data_fastcgi_copy(const data_unset *s) {
 	data_fastcgi *src = (data_fastcgi *)s;
 	data_fastcgi *ds = data_fastcgi_init();
 
-	buffer_copy_string_buffer(ds->key, src->key);
-	buffer_copy_string_buffer(ds->host, src->host);
+	buffer_copy_buffer(ds->key, src->key);
+	buffer_copy_buffer(ds->host, src->host);
 	ds->is_index_key = src->is_index_key;
 	return (data_unset *)ds;
 }
