@@ -9,6 +9,14 @@
 # define __USE_GNU /* a hack in my eyes, <fcntl.h> F_SETSIG should work with _GNU_SOURCE */
 #endif
 
+#ifdef __GNUC__
+# define LI_NORETURN __attribute__((noreturn))
+#else
+# define LI_NORETURN
+#endif
+
+#define UNUSED(x) ( (void)(x) )
+
 #define BV(x) (1 << x)
 
 #define INET_NTOP_CACHE_MAX 4
@@ -21,7 +29,10 @@
  * 64kB (no real reason, just a guess)
  */
 #define BUFFER_MAX_REUSE_SIZE  (4 * 1024)
-#define MAX_READ_LIMIT (4*1024*1024)
+
+/* both should be way smaller than SSIZE_MAX :) */
+#define MAX_READ_LIMIT (256*1024)
+#define MAX_WRITE_LIMIT (256*1024)
 
 /**
  * max size of the HTTP request header
